package com.bcxin.sync.common.utils;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.sync.common.exception.GetResponseException;
import com.bcxin.sync.dtos.ivm.ChannelDto;
import com.bcxin.sync.dtos.ivm.DeviceDto;
import com.bcxin.sync.service.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
public class HuaweiIvmUtil {

    /**
     * description：获取华为云IVM账户的AccessToken
     * author：linchunpeng
     * date：2025/5/27
     */
    public static String getAccessToken(String host, String userId, String ak, String sk, RedisCache redisCache) {
        log.info("获取华为云IVM账户的token");
        String key = "data-sync:huawei:ivm:access-token:" + userId;
        if (redisCache != null) {
            Object value = redisCache.getCacheObject(key);
            if (value != null) {
                return value.toString();
            }
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("ak", ak);
        paramMap.put("sk", sk);
        paramMap.put("force_update", true);

        String result = HttpUtil.post(host.concat("/v2/").concat(userId).concat("/enterprises/access-token"), JSONObject.toJSONString(paramMap), null, false);
        log.info("获取华为云IVM账户的token，结果：{}", result);
        if (result.equals("fail")) {
            //获取失败
            throw new GetResponseException(500, "登录失败，fail");
        }
        JSONObject resultObj = JSONObject.parseObject(result);
        if (resultObj.containsKey("access_token")) {
            //获取成功
            String token = resultObj.getString("access_token");
            if (redisCache != null) {
                redisCache.setCacheObject(key, token, 30, TimeUnit.MINUTES);
            }
            return token;
        } else {
            //获取失败
            throw new GetResponseException(500, "登录失败，" + resultObj.getString("msg"));
        }
    }


    /**
     * description：获取华为云IVM设备列表
     * author：linchunpeng
     * date：2025/5/27
     */
    public static List<DeviceDto> getDeviceList(String host, String userId, String accessToken) {
        log.info("获取华为云IVM设备列表");
        List<DeviceDto> resultList = new ArrayList<>();

        String result = HttpUtil.get(host.concat("/v1/").concat(userId).concat("/devices?limit=1000&offset=0"),
                getHeaderMap(accessToken), false);
        log.info("获取华为云IVM设备列表，结果：{}", result);
        if (!result.equals("fail")) {
            JSONObject resultObj = JSONObject.parseObject(result);
            if (resultObj.containsKey("total") && resultObj.getInteger("total") > 0) {
                //获取成功
                int total = resultObj.getInteger("total");
                resultList.addAll(resultObj.getJSONArray("devices").toJavaList(DeviceDto.class));
                if (total > 1000) {
                    //超过1页，需要分页获取
                    int page = total/1000;
                    for (int i = 1; i <= page; i++) {
                        String result_offset = HttpUtil.get(host.concat("/v1/").concat(userId).concat("/devices?limit=1000&offset=").concat(""+i)
                                , getHeaderMap(accessToken), false);
                        if (!result_offset.equals("fail")) {
                            JSONObject resultObj_offset = JSONObject.parseObject(result_offset);
                            if (resultObj_offset.containsKey("total") && resultObj_offset.getInteger("total") > 0) {
                                resultList.addAll(resultObj_offset.getJSONArray("devices").toJavaList(DeviceDto.class));
                            }
                        }
                    }
                }
            }
        }
        log.info("获取华为云IVM设备列表，结果对象：{}", JSONObject.toJSONString(resultList));
        return resultList;
    }


    /**
     * description：根据设备id，获取华为云IVM通道信息
     * author：linchunpeng
     * date：2025/5/27
     */
    public static ChannelDto getChannelByDeviceId(String host, String userId, String accessToken, String deviceId) {
        log.info("根据设备id，获取华为云IVM通道信息，设备id：{}", deviceId);

        String result = HttpUtil.get(host.concat("/v1/").concat(userId).concat("/channels?device_id=").concat(deviceId),
                getHeaderMap(accessToken), false);
        log.info("根据设备id，获取华为云IVM通道信息，设备id：{}，结果：{}", deviceId, result);
        if (!result.equals("fail")) {
            JSONObject resultObj = JSONObject.parseObject(result);
            if (resultObj.containsKey("total") && resultObj.getInteger("total") > 0) {
                //获取成功
                List<ChannelDto> channelDtoList = resultObj.getJSONArray("channels").toJavaList(ChannelDto.class);
                log.info("根据设备id，获取华为云IVM通道信息，设备id：{}，结果对象：{}", deviceId, JSONObject.toJSONString(channelDtoList));
                return channelDtoList.get(0);
            }
        }
        return new ChannelDto();
    }

    /**
     * description：根据设备id，通道id，批量删除路线计划
     * author：linchunpeng
     * date：2025/5/27
     */
    public static boolean deleteDeviceRecordPlan(String host, String userId, String accessToken, String deviceId, String channelId) {
        log.info("根据设备id，通道id，批量删除路线计划，设备id：{}，通道id：{}", deviceId, channelId);
        Map<String, Object> paramMap = new HashMap<>();
        List<Map<String, Object>> channelList = new ArrayList<>();
        Map<String, Object> channel = new HashMap<>();
        channel.put("device_id", deviceId);
        channel.put("channel_id", channelId);
        channelList.add(channel);
        paramMap.put("channels", channelList);

        String result = HttpUtil.post(host.concat("/v1/").concat(userId).concat("/devices/channels/record-plan"),
                JSONObject.toJSONString(paramMap), getHeaderMap(accessToken), false);
        log.info("根据设备id：{}，通道id：{}，批量删除路线计划，结果：{}", deviceId, channelId, result);
        if (!result.equals("fail")) {
            JSONObject resultObj = JSONObject.parseObject(result);
            if (resultObj.containsKey("failed_num") && resultObj.getInteger("failed_num") == 0) {
                //删除成功
                return true;
            }
        }
        return false;
    }


    /**
     * description：根据设备id，设置通道录像计划
     * author：linchunpeng
     * date：2025/5/27
     */
    public static void setDeviceRecordPlan(String host, String userId, String accessToken, String deviceId, List<Date> startTimeList, List<Date> endTimeList) {
        log.info("根据设备id，设置通道录像计划，设备id：{}，开始时间：{}，结束时间：{}", deviceId, startTimeList, endTimeList);
        ChannelDto channelDto = getChannelByDeviceId(host, userId, accessToken, deviceId);
        String channelId = channelDto.getChannel_id();
        log.info("根据设备id：{}，获取通道id：{}", deviceId, channelId);
        if (StringUtils.isNotBlank(channelId)) {
            boolean deleteSuccess = deleteDeviceRecordPlan(host, userId, accessToken, deviceId, channelId);
            log.info("根据设备id：{}，通道id：{}，批量删除路线计划，结果：{}", deviceId, channelId, deleteSuccess);

            if (CollectionUtil.isNotEmpty(startTimeList)
                    && CollectionUtil.isNotEmpty(endTimeList)
                    && startTimeList.size() == endTimeList.size()) {
                log.info("根据设备id：{}，获取通道id：{}，计划录像时间不为空", deviceId, channelId);
                for (int i = 1; i < startTimeList.size(); i++) {
                    Date nextStartTime = startTimeList.get(i);
                    Date endTime = endTimeList.get(i-1);
                    if (nextStartTime.before(endTime)) {
                        nextStartTime = endTime;
                        startTimeList.set(i, nextStartTime);
                    }
                }
                for (int i = startTimeList.size() - 1; i >= 0; i--) {
                    Date startTime = startTimeList.get(i);
                    Date endTime = endTimeList.get(i);
                    if (endTime.before(startTime)) {
                        startTimeList.remove(i);
                        endTimeList.remove(i);
                    }
                }

                Map<String, Object> paramMap = new HashMap<>();
                List<Map<String, Object>> planList = new ArrayList<>();
                Map<String, Object> plan = new HashMap<>();
                plan.put("device_id", deviceId);
                plan.put("channel_id", channelId);
                plan.put("all_day", false);

                List<Map<String, Object>> timeSectionList = new ArrayList<>();
                for (int i = 0; i <startTimeList.size(); i++) {
                    Date startTime = startTimeList.get(i);
                    Date endTime = endTimeList.get(i);
                    if (startTime.after(endTime)) {
                        throw new GetResponseException(500, String.format("根据设备id，设置通道录像计划，设备id：%s，开始时间：%s，结束时间：%s，结束时间不能小于开始时间", deviceId, startTimeList, endTimeList));
                    }
                    int startDayOfWeek = DateUtil.getWeekDayOfDate(startTime);
                    int endDayOfWeek = DateUtil.getWeekDayOfDate(endTime);

                    String startStr = cn.hutool.core.date.DateUtil.format(startTime, "HH:mm:ss");
                    String endStr = cn.hutool.core.date.DateUtil.format(endTime, "HH:mm:ss");

                    if (startDayOfWeek == endDayOfWeek) {
                        //开始和结束在同一天
                        Map<String, Object> timeSection = new HashMap<>();
                        timeSection.put("day_of_week", startDayOfWeek);
                        timeSection.put("start_time", startStr);
                        timeSection.put("end_time", endStr);
                        timeSectionList.add(timeSection);
                    } else {
                        //开始和结束跨天
                        for (int d = startDayOfWeek; d <= endDayOfWeek; d++) {
                            Map<String, Object> timeSection = new HashMap<>();
                            timeSection.put("day_of_week", d);
                            if (d == startDayOfWeek) {
                                //第一天
                                timeSection.put("start_time", startStr);
                                timeSection.put("end_time", "23:59:59");
                            } else if (d == endDayOfWeek) {
                                //最后一天
                                timeSection.put("start_time", "00:00:00");
                                timeSection.put("end_time", endStr);
                            } else {
                                //中间天
                                timeSection.put("start_time", "00:00:00");
                                timeSection.put("end_time", "23:59:59");
                            }
                            timeSectionList.add(timeSection);
                        }
                    }
                }

                plan.put("time_sections", timeSectionList);
                planList.add(plan);
                paramMap.put("plans", planList);
                log.info("根据设备id，设置通道录像计划，设备id：{}，参数plans：{}", deviceId, planList);
                String result = HttpUtil.put(host.concat("/v1/").concat(userId).concat("/devices/channels/record-plan"),
                        JSONObject.toJSONString(paramMap), getHeaderMap(accessToken), false);
                log.info("根据设备id，设置通道录像计划，设备id：{}，结果：{}", deviceId, result);
                if (!result.equals("fail")) {
                    JSONObject resultObj = JSONObject.parseObject(result);
                    if (resultObj.containsKey("failed_num") && resultObj.getInteger("failed_num") == 0) {
                        //设置成功
                        log.info("根据设备id，设置通道录像计划，设备id：{}，设置成功", deviceId);
                    }
                } else {
                    log.info("根据设备id，设置通道录像计划，设备id：{}，设置失败", deviceId);
                }
            } else {
                log.info("根据设备id：{}，获取通道id：{}，计划录像时间为空", deviceId, channelId);
            }
        }
    }


    /**
     * description：获取调用接口的头部参数与值
     * author：linchunpeng
     * date：2025/5/27
     */
    public static Map<String, String> getHeaderMap(String accessToken) {
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("Access-Token", accessToken);
        return headerMap;
    }

    public static void main(String[] args) {
        String host = "https://api-ivm.myhuaweicloud.com";
        String userId = "5994908220250519164352";
        String ak = "ae1f71705d4ad910450c106e4229d0a3";
        String sk = "48b057c1a009a0bf80749a832294298844c2fccbc796c8121c9425399fc0cf0f";
        String token = getAccessToken(host, userId, ak, sk, null);
        System.out.println("token="+token);
        List<DeviceDto> deviceList = getDeviceList(host, userId, token);
        System.out.println("deviceList="+JSONObject.toJSONString(deviceList));
//        ChannelDto channelDto = getChannelByDeviceId(host, userId, token, deviceList.get(0).getDevice_id());
//        System.out.println("channelDto="+JSONObject.toJSONString(channelDto));

        List<Date> startTimeList = new ArrayList<>(Arrays.asList(
                cn.hutool.core.date.DateUtil.parse("2025-05-29 10:15:00", "yyyy-MM-dd HH:mm:ss"),
                cn.hutool.core.date.DateUtil.parse("2025-05-29 11:15:00", "yyyy-MM-dd HH:mm:ss")));
        List<Date> endTimeList = new ArrayList<>(Arrays.asList(
                cn.hutool.core.date.DateUtil.parse("2025-05-29 12:05:00", "yyyy-MM-dd HH:mm:ss"),
                cn.hutool.core.date.DateUtil.parse("2025-05-29 12:00:00", "yyyy-MM-dd HH:mm:ss")));
        setDeviceRecordPlan(host, userId, token, deviceList.get(0).getDevice_id(), startTimeList, endTimeList);
//        System.out.println(cn.hutool.core.date.DateUtil.format(cn.hutool.core.date.DateUtil.parse("2025-05-26 12:33:11"), "HH:mm:ss"));
    }
}
